# This file is auto-generated by hatch-gradle-version. Do not edit.

GRADLE_VERSION = "1.5.3"
FULL_VERSION = "1.5.3.1.4.dev0"

ARCHIVES_BASE_NAME = "lapisworks"
CCA_VERSION = "5.2.3"
FABRIC_KOTLIN_VERSION = "1.13.4+kotlin.2.2.0"
FABRIC_VERSION = "0.92.6+1.20.1"
HEXCASTING_VERSION = "0.11.2-pre-716"
LOADER_VERSION = "0.16.14"
LOOM_VERSION = "1.11-SNAPSHOT"
MAVEN_GROUP = "com.luxof.lapisworks"
MINECRAFT_VERSION = "1.20.1"
MOD_VERSION = "1.5.3"
ORG_GRADLE_JVMARGS = "-Xmx1G"
ORG_GRADLE_PARALLEL = "true"
TRINKETS_VERSION = "3.7.2"
YARN_MAPPINGS = "1.20.1+build.10"
